@@ -16,13 +16,13 @@
 int
 get_gid(const char *gidstr, gid_t *gid)
 {
+	char       *end;
 	long long  val;
-	char *endptr;
 
 	errno = 0;
-	val = strtoll(gidstr, &endptr, 10);
+	val = strtoll(gidstr, &end, 10);
 	if (   ('\0' == *gidstr)
-	    || ('\0' != *endptr)
+	    || ('\0' != *end)
 	    || (0 != errno)
 	    || (/*@+longintegral@*/val != (gid_t)val)/*@=longintegral@*/) {
 		return -1;
