@@ -14,6 +14,7 @@
 #include <stdio.h>
 #include <sys/types.h>
 #include <grp.h>
+#include <string.h>
 
 #include "alloc.h"
 #include "defines.h"
@@ -27,14 +28,11 @@
  *	list() converts the comma-separated list of member names into
  *	an array of character pointers.
  *
- *	WARNING: I profiled this once with and without strchr() calls
- *	and found that using a register variable and an explicit loop
- *	works best.  For large /etc/group files, this is a major win.
- *
  * FINALLY added dynamic allocation.  Still need to fix sgetsgent().
  *  --marekm
  */
-static char **list (char *s)
+static char **
+list(char *s)
 {
 	static char **members = NULL;
 	static size_t size = 0;	/* max members + 1 */
@@ -54,13 +52,7 @@ static char **list (char *s)
 		}
 		if (!s || s[0] == '\0')
 			break;
-		members[i++] = s;
-		while (('\0' != *s) && (',' != *s)) {
-			s++;
-		}
-		if ('\0' != *s) {
-			*s++ = '\0';
-		}
+		members[i++] = strsep(&s, ",");
 	}
 	members[i] = NULL;
 	return members;
@@ -94,20 +86,15 @@ struct group *sgetgrent (const char *buf)
 		*cp = '\0';
 	}
 
-	for (cp = grpbuf, i = 0; (i < NFIELDS) && (NULL != cp); i++) {
-		grpfields[i] = cp;
-		cp = strchr (cp, ':');
-		if (NULL != cp) {
-			*cp = '\0';
-			cp++;
-		}
-	}
+	for (cp = grpbuf, i = 0; (i < NFIELDS) && (NULL != cp); i++)
+		grpfields[i] = strsep(&cp, ":");
+
 	if (i < (NFIELDS - 1) || *grpfields[2] == '\0' || cp != NULL) {
 		return NULL;
 	}
 	grent.gr_name = grpfields[0];
 	grent.gr_passwd = grpfields[1];
-	if (get_gid (grpfields[2], &grent.gr_gid) == 0) {
+	if (get_gid(grpfields[2], &grent.gr_gid) == -1) {
 		return NULL;
 	}
 	grent.gr_mem = list (grpfields[3]);
