@@ -27,6 +27,7 @@
 #include <pwd.h>
 
 #include "alloc.h"
+#include "atoi/str2i.h"
 #include "defines.h"
 #include "memzero.h"
 #include "prototypes.h"
@@ -170,14 +171,14 @@ static int new_fields (void)
 
 	SNPRINTF(buf, "%ld", mindays);
 	change_field (buf, sizeof buf, _("Minimum Password Age"));
-	if (   (getlong(buf, &mindays) == -1)
+	if (   (str2sl(&mindays, buf) == -1)
 	    || (mindays < -1)) {
 		return 0;
 	}
 
 	SNPRINTF(buf, "%ld", maxdays);
 	change_field (buf, sizeof buf, _("Maximum Password Age"));
-	if (   (getlong(buf, &maxdays) == -1)
+	if (   (str2sl(&maxdays, buf) == -1)
 	    || (maxdays < -1)) {
 		return 0;
 	}
@@ -200,14 +201,14 @@ static int new_fields (void)
 
 	SNPRINTF(buf, "%ld", warndays);
 	change_field (buf, sizeof buf, _("Password Expiration Warning"));
-	if (   (getlong(buf, &warndays) == -1)
+	if (   (str2sl(&warndays, buf) == -1)
 	    || (warndays < -1)) {
 		return 0;
 	}
 
 	SNPRINTF(buf, "%ld", inactdays);
 	change_field (buf, sizeof buf, _("Password Inactive"));
-	if (   (getlong(buf, &inactdays) == -1)
+	if (   (str2sl(&inactdays, buf) == -1)
 	    || (inactdays < -1)) {
 		return 0;
 	}
@@ -396,7 +397,7 @@ static void process_flags (int argc, char **argv)
 			break;
 		case 'I':
 			Iflg = true;
-			if (   (getlong(optarg, &inactdays) == -1)
+			if (   (str2sl(&inactdays, optarg) == -1)
 			    || (inactdays < -1)) {
 				fprintf (stderr,
 				         _("%s: invalid numeric argument '%s'\n"),
@@ -409,7 +410,7 @@ static void process_flags (int argc, char **argv)
 			break;
 		case 'm':
 			mflg = true;
-			if (   (getlong(optarg, &mindays) == -1)
+			if (   (str2sl(&mindays, optarg) == -1)
 			    || (mindays < -1)) {
 				fprintf (stderr,
 				         _("%s: invalid numeric argument '%s'\n"),
@@ -419,7 +420,7 @@ static void process_flags (int argc, char **argv)
 			break;
 		case 'M':
 			Mflg = true;
-			if (   (getlong(optarg, &maxdays) == -1)
+			if (   (str2sl(&maxdays, optarg) == -1)
 			    || (maxdays < -1)) {
 				fprintf (stderr,
 				         _("%s: invalid numeric argument '%s'\n"),
@@ -433,7 +434,7 @@ static void process_flags (int argc, char **argv)
 			break;
 		case 'W':
 			Wflg = true;
-			if (   (getlong(optarg, &warndays) == -1)
+			if (   (str2sl(&warndays, optarg) == -1)
 			    || (warndays < -1)) {
 				fprintf (stderr,
 				         _("%s: invalid numeric argument '%s'\n"),
