@@ -9,9 +9,9 @@
 
 /* Test program for the subid freeing routine */
 
-const char *Prog;
+static const char Prog[] = "free_subid_range";
 
-void usage(void)
+static void usage(void)
 {
 	fprintf(stderr, "Usage: %s [-g] user start count\n", Prog);
 	fprintf(stderr, "    Release a user's subuid (or with -g, subgid) range\n");
@@ -25,7 +25,6 @@ int main(int argc, char *argv[])
 	struct subordinate_range range;
 	bool group = false;   // get subuids by default
 
-	Prog = Basename (argv[0]);
 	log_set_progname(Prog);
 	log_set_logfd(stderr);
 	while ((c = getopt(argc, argv, "g")) != EOF) {
