@@ -24,6 +24,8 @@
 #include <unistd.h>
 #include <grp.h>
 #include <getopt.h>
+
+#include "attr.h"
 #include "nscd.h"
 #include "sssd.h"
 #include "prototypes.h"
@@ -221,7 +223,7 @@ int main (int argc, char **argv)
 	return 0;
 }
 #else				/* !SHADOWGRP */
-int main (int unused(argc), char **argv)
+int main (MAYBE_UNUSED int argc, char **argv)
 {
 	fprintf (stderr,
 		 "%s: not configured for shadow group support.\n", argv[0]);
