@@ -16,6 +16,11 @@
 #include <signal.h>
 #include <stdio.h>
 #include <sys/ioctl.h>
+#include <sys/types.h>
+
+#include "agetpass.h"
+#include "alloc.h"
+#include "attr.h"
 #include "defines.h"
 #include "getdef.h"
 #include "prototypes.h"
@@ -24,38 +29,42 @@
 #include "exitcodes.h"
 #include "shadowlog.h"
 
+
 /*
  * Global variables
  */
 static const char Prog[] = "sulogin";
 
-static char pass[BUFSIZ];
-
-static struct passwd pwent;
 
 extern char **newenvp;
-extern size_t newenvc;
 
 #ifndef	ALARM
 #define	ALARM	60
 #endif
 
-/* local function prototypes */
-static void catch_signals (int);
 
-static void catch_signals (unused int sig)
+static void catch_signals (int);
+static int pw_entry(const char *name, struct passwd *pwent);
+
+
+static void catch_signals (MAYBE_UNUSED int sig)
 {
 	_exit (1);
 }
 
- /*ARGSUSED*/ int main (int argc, char **argv)
+
+int
+main(int argc, char *argv[])
 {
+	int            err = 0;
+	char           **envp = environ;
+	TERMIO         termio;
+	struct passwd  pwent = {};
+	bool           done;
 #ifndef USE_PAM
-	const char *env;
-#endif				/* !USE_PAM */
-	char **envp = environ;
-	TERMIO termio;
-	int err = 0;
+	const char     *env;
+#endif
+
 
 	tcgetattr (0, &termio);
 	termio.c_iflag |= (ICRNL | IXON);
@@ -69,18 +78,16 @@ static void catch_signals (unused int sig)
 	(void) bindtextdomain (PACKAGE, LOCALEDIR);
 	(void) textdomain (PACKAGE);
 
-	initenv ();
+	initenv();
 	if (argc > 1) {
-		close (0);
-		close (1);
-		close (2);
+		close(0);
+		close(1);
+		close(2);
 
-		if (open (argv[1], O_RDWR) >= 0) {
-			dup (0);
-			dup (0);
-		} else {
-			exit (1);
-		}
+		if (open(argv[1], O_RDWR) == -1)
+			exit(1);
+		dup(0);
+		dup(0);
 	}
 	if (access (PASSWD_FILE, F_OK) == -1) {	/* must be a password file! */
 		(void) puts (_("No password file"));
@@ -120,15 +127,14 @@ static void catch_signals (unused int sig)
 	(void) signal (SIGALRM, catch_signals);	/* exit if the timer expires */
 	(void) alarm (ALARM);		/* only wait so long ... */
 
-	while (true) {		/* repeatedly get login/password pairs */
-		char *cp;
-		pw_entry ("root", &pwent);	/* get entry from password file */
-		if (pwent.pw_name == NULL) {
+	do {			/* repeatedly get login/password pairs */
+		char *pass;
+		if (pw_entry("root", &pwent) == -1) {	/* get entry from password file */
 			/*
 			 * Fail secure
 			 */
-			(void) puts (_("No password entry for 'root'"));
-			exit (1);
+			(void) puts(_("No password entry for 'root'"));
+			exit(1);
 		}
 
 		/*
@@ -137,7 +143,7 @@ static void catch_signals (unused int sig)
 		 */
 
 		/* get a password for root */
-		cp = agetpass (_(
+		pass = agetpass (_(
 "\n"
 "Type control-d to proceed with normal startup,\n"
 "(or give root password for system maintenance):"));
@@ -147,25 +153,24 @@ static void catch_signals (unused int sig)
 		 * it will work with standard getpass() (no NULL on EOF).
 		 * --marekm
 		 */
-		if ((NULL == cp) || ('\0' == *cp)) {
-			erase_pass (cp);
+		if ((NULL == pass) || ('\0' == *pass)) {
+			erase_pass (pass);
 			(void) puts ("");
 #ifdef	TELINIT
 			execl (PATH_TELINIT, "telinit", RUNLEVEL, (char *) NULL);
 #endif
 			exit (0);
 		}
-		STRFCPY (pass, cp);
-		erase_pass (cp);
 
-		if (valid (pass, &pwent)) {	/* check encrypted passwords ... */
-			break;	/* ... encrypted passwords matched */
+		done = valid(pass, &pwent);
+		erase_pass (pass);
+
+		if (!done) {	/* check encrypted passwords ... */
+			/* ... encrypted passwords did not match */
+			sleep (2);
+			(void) puts (_("Login incorrect"));
 		}
-
-		sleep (2);
-		(void) puts (_("Login incorrect"));
-	}
-	memzero (pass, sizeof pass);
+	} while (!done);
 	(void) alarm (0);
 	(void) signal (SIGALRM, SIG_DFL);
 	environ = newenvp;	/* make new environment active */
@@ -178,3 +183,35 @@ static void catch_signals (unused int sig)
 	return ((err == ENOENT) ? E_CMD_NOTFOUND : E_CMD_NOEXEC);
 }
 
+
+static int
+pw_entry(const char *name, struct passwd *pwent)
+{
+	struct spwd    *spwd;
+	struct passwd  *passwd;
+
+	if (!(passwd = getpwnam(name)))  /* local, no need for xgetpwnam */
+		return -1;
+
+	free(pwent->pw_name);
+	pwent->pw_name = xstrdup(passwd->pw_name);
+	pwent->pw_uid = passwd->pw_uid;
+	pwent->pw_gid = passwd->pw_gid;
+	free(pwent->pw_gecos);
+	pwent->pw_gecos = xstrdup(passwd->pw_gecos);
+	free(pwent->pw_dir);
+	pwent->pw_dir = xstrdup(passwd->pw_dir);
+	free(pwent->pw_shell);
+	pwent->pw_shell = xstrdup(passwd->pw_shell);
+#if !defined(AUTOSHADOW)
+	/* local, no need for xgetspnam */
+	if ((spwd = getspnam(name))) {
+		free(pwent->pw_passwd);
+		pwent->pw_passwd = xstrdup(spwd->sp_pwdp);
+		return 0;
+	}
+#endif
+	free(pwent->pw_passwd);
+	pwent->pw_passwd = xstrdup(passwd->pw_passwd);
+	return 0;
+}
